#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class ForeignBornDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "foreignborn"
    RAW_TABLE_NAME = 'B05002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "total",
        "002E": "native",
        "013E": "foreign_born"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class HouseholdIncomeDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "householdincome"
    RAW_TABLE_NAME = 'B19001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': "universe",
        '002E': "10_and_under",
        '003E': "10_15",
        '004E': "15_20",
        '005E': "20_25",
        '006E': "25_30",
        '007E': "30_35",
        '008E': "35_40",
        '009E': "40_45",
        '010E': "45_50",
        '011E': "50_60",
        '012E': "60_75",
        '013E': "75_100",
        '014E': "100_125",
        '015E': "125_150",
        '016E': "150_200",
        '017E': "200_and_up"
    })


@register_downloader
class HouseholdIncomeLatinoDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomelatino"
    RAW_TABLE_NAME = 'B19001I'


@register_downloader
class HouseholdIncomeWhiteDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomewhite"
    RAW_TABLE_NAME = 'B19001H'


@register_downloader
class HouseholdIncomeBlackDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomeblack"
    RAW_TABLE_NAME = 'B19001B'


@register_downloader
class HouseholdIncomeAsianDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomeasian"
    RAW_TABLE_NAME = 'B19001D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
DOWNLOADERS = []


def register_downloader(cls):
    """
    A decorator to register new classes with the application.
    """
    DOWNLOADERS.append(cls)
    return cls



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Command-line interface.
"""
import click
from census_data_downloader import DOWNLOADERS
DOWNLOADERS_LOOKUP = dict((k.PROCESSED_TABLE_NAME, k) for k in DOWNLOADERS)


@click.group(help="Download Census data and reformat it for humans")
@click.argument(
    "table",
    nargs=1,
    required=True
)
@click.option(
    "--data-dir",
    nargs=1,
    default="./",
    help="The folder where you want to download the data"
)
@click.option(
    "--year",
    default=None,
    type=int,
    help="The years of data to download. By default it gets only the latest year. Submit 'all' to get every year."
)
@click.option(
    '--force',
    is_flag=True,
    help="Force the downloading of the data"
)
@click.pass_context
def cmd(ctx, table, data_dir="./", year=None, force=False):
    ctx.ensure_object(dict)
    ctx.obj['table'] = table
    ctx.obj['data_dir'] = data_dir
    ctx.obj['year'] = year
    ctx.obj['force'] = force
    try:
        klass = DOWNLOADERS_LOOKUP[ctx.obj['table']]
        ctx.obj['klass'] = klass
        ctx.obj['runner'] = klass(
            data_dir=data_dir,
            years=year,
            force=force
        )
    except KeyError:
        click.ClickException("Table not found")


@cmd.command(help="Download nationwide data")
@click.pass_context
def nationwide(ctx):
    ctx.obj['runner'].download_nationwide()


@cmd.command(help="Download states")
@click.pass_context
def states(ctx):
    ctx.obj['runner'].download_states()


@cmd.command(help="Download Congressional districts")
@click.pass_context
def congressionaldistricts(ctx):
    ctx.obj['runner'].download_congressional_districts()


@cmd.command(help="Download counties in all states")
@click.pass_context
def counties(ctx):
    ctx.obj['runner'].download_counties()


@cmd.command(help="Download Census-designated places")
@click.pass_context
def places(ctx):
    ctx.obj['runner'].download_places()


@cmd.command(help="Download Census tracts in provided state")
@click.argument(
    "state",
    nargs=1,
    required=True
)
@click.pass_context
def tracts(ctx, state):
    ctx.obj['runner'].download_tracts(state)


@cmd.command(help="Download statehouse districts in provided state")
@click.argument(
    "statelegislativedistricts",
    nargs=1,
    required=True
)
@click.pass_context
def statelegislativedistricts(ctx, state):
    ctx.obj['runner'].download_state_legislative_districts_upper(state)
    ctx.obj['runner'].download_state_legislative_districts_lower(state)


@cmd.command(help="Download all datasets that cover full USA")
@click.pass_context
def usa(ctx):
    ctx.obj['runner'].download_usa()


@cmd.command(help="Download everything from everywhere")
@click.pass_context
def everything(ctx):
    ctx.obj['runner'].download_everything()


if __name__ == '__main__':
    cmd()



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class EmploymentStatusDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "employmentstatus"
    RAW_TABLE_NAME = 'B23025'
    YEAR_LIST = (
        2017,
        2016,
        2015,
        2014,
        2013,
        2012,
        2011
    )
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': 'total_population_16over',
        '002E': 'in_labor_force',
        '003E': 'civilian',
        '004E': 'civilian_employed',
        '005E': 'civilian_unemployed',
        '006E': 'armed_forces',
        '007E': 'not_in_labor_force'
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class MobilityDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "mobility"
    RAW_TABLE_NAME = 'B07003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "total",
        "004E": "same_house",
        "007E": "moved_within_county",
        "010E": "moved_from_different_county_in_same_state",
        "013E": "moved_from_different_state",
        "016E": "moved_from_abroad",
    })


@register_downloader
class MobilityBySexDownloader(MobilityDownloader):
    PROCESSED_TABLE_NAME = "mobilitybysex"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "total",
        "002E": "male_total",
        "003E": "female_total",
        "004E": "total_same_house",
        "005E": "male_same_house",
        "006E": "female_same_house",
        "007E": "total_moved_within_county",
        "008E": "male_moved_within_county",
        "009E": "female_moved_within_county",
        "010E": "total_moved_from_different_county_in_same_state",
        "011E": "male_moved_from_different_county_in_same_state",
        "012E": "female_moved_from_different_county_in_same_state",
        "013E": "total_moved_from_different_state",
        "014E": "male_moved_from_different_state",
        "015E": "female_moved_from_different_state",
        "016E": "total_moved_from_abroad",
        "017E": "male_moved_from_abroad",
        "018E": "female_moved_from_abroad"
    })


@register_downloader
class MobilityWhiteDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "mobilitywhite"
    RAW_TABLE_NAME = 'B07004H'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "total",
        "002E": "same_house",
        "003E": "moved_within_county",
        "004E": "moved_from_different_county_in_same_state",
        "005E": "moved_from_different_state",
        "006E": "moved_from_abroad"
    })


@register_downloader
class MobilityBlackDownloader(MobilityWhiteDownloader):
    PROCESSED_TABLE_NAME = "mobilityblack"
    RAW_TABLE_NAME = 'B07004B'


@register_downloader
class MobilityAsianDownloader(MobilityWhiteDownloader):
    PROCESSED_TABLE_NAME = "mobilityasian"
    RAW_TABLE_NAME = 'B07004D'


@register_downloader
class MobilityLatinoDownloader(MobilityWhiteDownloader):
    PROCESSED_TABLE_NAME = "mobilitylatino"
    RAW_TABLE_NAME = 'B07004I'



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class RaceDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "race"
    RAW_TABLE_NAME = 'B03002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': "total",
        '003E': "white_alone",
        '004E': "black_alone",
        '005E': "american_indian_and_alaska_native",
        '006E': "asian_alone",
        '007E': "native_hawaiian_and_pacific_islander",
        '008E': "other_alone",
        '009E': "two_or_more_races",
        '012E': "latino_alone"
    })

    def _process_raw_data(self, *args, **kwargs):
        df = super()._process_raw_data(*args, **kwargs)
        # Our custom race groups
        df['asian_all'] = df['asian_alone'] + df['native_hawaiian_and_pacific_islander']
        df['other_all'] = df[[
            'american_indian_and_alaska_native',
            'other_alone',
            'two_or_more_races'
        ]].sum(axis=1)
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class LanguageDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = 'language'
    RAW_TABLE_NAME = 'B16001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': 'total',
        '002E': 'only_english',
        '003E': 'total_spanish',
        '004E': 'spanish_and_english_very_well',
        '005E': 'spanish_and_english_less_than_very_well',
        '006E': 'total_french',
        '007E': 'french_and_english_very_well',
        '008E': 'french_and_english_less_than_very_well',
        '009E': 'total_haitian',
        '010E': 'haitian_and_english_very_well',
        '011E': 'haitian_and_english_less_than_very_well',
        '012E': 'total_italian',
        '013E': 'italian_and_english_very_well',
        '014E': 'italian_and_english_less_than_very_well',
        '015E': 'total_portuguese',
        '016E': 'portuguese_and_english_very_well',
        '017E': 'portuguese_and_english_less_than_very_well',
        '018E': 'total_german',
        '019E': 'german_and_english_very_well',
        '020E': 'german_and_english_less_than_very_well',
        '021E': 'total_other_germanic',
        '022E': 'other_germanic_and_english_very_well',
        '023E': 'other_germanic_and_english_less_than_very_well',
        '024E': 'total_greek',
        '025E': 'greek_and_english_very_well',
        '026E': 'greek_and_english_less_than_very_well',
        '027E': 'total_russian',
        '028E': 'russian_and_english_very_well',
        '029E': 'russian_and_english_less_than_very_well',
        '030E': 'total_polish',
        '031E': 'polish_and_english_very_well',
        '032E': 'polish_and_english_less_than_very_well',
        '033E': 'total_serbo_croatian',
        '034E': 'serbo_croatian_and_english_very_well',
        '035E': 'serbo_croatian_and_english_less_than_very_well',
        '036E': 'total_other_slavic',
        '037E': 'other_slavic_and_english_very_well',
        '038E': 'other_slavic_and_english_less_than_very_well',
        '039E': 'total_armenian',
        '040E': 'armenian_and_english_very_well',
        '041E': 'armenian_and_english_less_than_very_well',
        '042E': 'total_persian',
        '043E': 'persian_and_english_very_well',
        '044E': 'persian_and_english_less_than_very_well',
        '045E': 'total_gujarati',
        '046E': 'gujarati_and_english_very_well',
        '047E': 'gujarati_and_english_less_than_very_well',
        '048E': 'total_hindi',
        '049E': 'hindi_and_english_very_well',
        '050E': 'hindi_and_english_less_than_very_well',
        '051E': 'total_urdu',
        '052E': 'urdu_and_english_very_well',
        '053E': 'urdu_and_english_less_than_very_well',
        '054E': 'total_punjabi',
        '055E': 'punjabi_and_english_very_well',
        '056E': 'punjabi_and_english_less_than_very_well',
        '057E': 'total_bengali',
        '058E': 'bengali_and_english_very_well',
        '059E': 'bengali_and_english_less_than_very_well',
        '060E': 'total_other_indic',
        '061E': 'other_indic_and_english_very_well',
        '062E': 'other_indic_and_english_less_than_very_well',
        '063E': 'total_other_indoeuropean',
        '064E': 'other_indoeuropean_and_english_very_well',
        '065E': 'other_indoeuropean_and_english_less_than_very_well',
        '066E': 'total_telugu',
        '067E': 'telugu_and_english_very_well',
        '068E': 'telugu_and_english_less_than_very_well',
        '069E': 'total_tamil',
        '070E': 'tamil_and_english_very_well',
        '071E': 'tamil_and_english_less_than_very_well',
        '072E': 'total_other_dravidian',
        '073E': 'other_dravidian_and_english_very_well',
        '074E': 'other_dravidian_and_english_less_than_very_well',
        '075E': 'total_chinese',
        '076E': 'chinese_and_english_very_well',
        '077E': 'chinese_and_english_less_than_very_well',
        '078E': 'total_japanese',
        '079E': 'japanese_and_english_very_well',
        '080E': 'japanese_and_english_less_than_very_well',
        '081E': 'total_korean',
        '082E': 'korean_and_english_very_well',
        '083E': 'korean_and_english_less_than_very_well',
        '084E': 'total_hmong',
        '085E': 'hmong_and_english_very_well',
        '086E': 'hmong_and_english_less_than_very_well',
        '087E': 'total_vietnamese',
        '088E': 'vietnamese_and_english_very_well',
        '089E': 'vietnamese_and_english_less_than_very_well',
        '090E': 'total_khmer',
        '091E': 'khmer_and_english_very_well',
        '092E': 'khmer_and_english_less_than_very_well',
        '093E': 'total_other_tai_kadai',
        '094E': 'other_tai_kadai_and_english_very_well',
        '095E': 'other_tai_kadai_and_english_less_than_very_well',
        '096E': 'total_other_asia',
        '097E': 'other_asia_and_english_very_well',
        '098E': 'other_asia_and_english_less_than_very_well',
        '099E': 'total_tagalog',
        '100E': 'tagalog_and_english_very_well',
        '101E': 'tagalog_and_english_less_than_very_well',
        '102E': 'total_other_austronesian',
        '103E': 'other_austronesian_and_english_very_well',
        '104E': 'other_austronesian_and_english_less_than_very_well',
        '105E': 'total_arabic',
        '106E': 'arabic_and_english_very_well',
        '107E': 'arabic_and_english_less_than_very_well',
        '108E': 'total_hebrew',
        '109E': 'hebrew_and_english_very_well',
        '110E': 'hebrew_and_english_less_than_very_well',
        '111E': 'total_other_afroasiatic',
        '112E': 'other_afroasiatic_and_english_very_well',
        '113E': 'other_afroasiatic_and_english_less_than_very_well',
        '114E': 'total_other_western_africa',
        '115E': 'other_western_africa_and_english_very_well',
        '116E': 'other_western_africa_and_english_less_than_very_well',
        '117E': 'total_other_central_eastern_southern_africa',
        '118E': 'other_central_eastern_southern_africa_and_english_very_well',
        '119E': 'other_central_eastern_southern_africa_and_english_less_than_very_well',
        '120E': 'total_navajo',
        '121E': 'navajo_and_english_very_well',
        '122E': 'navajo_and_english_less_than_very_well',
        '123E': 'total_other_native_north_america',
        '124E': 'other_native_north_america_and_english_very_well',
        '125E': 'other_native_north_america_and_english_less_than_very_well',
        '126E': 'total_other_unspecified',
        '127E': 'other_unspecified_and_english_very_well',
        '128E': 'other_unspecified_and_english_less_than_very_well'
    })

    def _process_raw_data(self, *args, **kwargs):
        """
        Combine language counts to get total english/non-english speakers
        """
        df = super()._process_raw_data(*args, **kwargs)

        languages = [
            'spanish',
            'french',
            'haitian',
            'italian',
            'portuguese',
            'german',
            'other_germanic',
            'greek',
            'russian',
            'polish',
            'serbo_croatian',
            'other_slavic',
            'armenian',
            'persian',
            'gujarati',
            'hindi',
            'urdu',
            'punjabi',
            'bengali',
            'other_indic',
            'other_indoeuropean',
            'telugu',
            'tamil',
            'other_dravidian',
            'chinese',
            'japanese',
            'korean',
            'hmong',
            'vietnamese',
            'khmer',
            'other_tai_kadai',
            'other_asia',
            'tagalog',
            'other_austronesian',
            'arabic',
            'hebrew',
            'other_afroasiatic',
            'other_western_africa',
            'other_central_eastern_southern_africa',
            'navajo',
            'other_native_north_america',
            'other_unspecified'
        ]

        # English vs. no English totals
        df['total_english'] = df['only_english'] + df[[f'{l}_and_english_very_well' for l in languages]].sum(axis=1)
        df['total_english_less_than_very_well'] = df[
            [f'{l}_and_english_less_than_very_well' for l in languages]
        ].sum(axis=1)

        # Group into the four language groups defined by the Census
        # https://www.census.gov/topics/population/language-use/about.html
        # Calculate our custom groups (other than Spanish, which we already have)
        groupsets = collections.OrderedDict({
            "other_indo_european_group": [
                'french',
                'haitian',
                'italian',
                'portuguese',
                'german',
                'other_germanic',
                'greek',
                'russian',
                'polish',
                'serbo_croatian',
                'other_slavic',
                'armenian',
                'persian',
                'gujarati',
                'hindi',
                'urdu',
                'punjabi',
                'bengali',
                'other_indic',
                'other_indoeuropean',
                'telugu',
                'tamil',
                    'other_dravidian',
            ],
            "asian_and_pacific_island_group": [
                'chinese',
                'japanese',
                'korean',
                'hmong',
                'vietnamese',
                'khmer',
                'other_tai_kadai',
                'other_asia',
                'tagalog',
                'other_austronesian',
            ],
            "all_other_group": [
                'arabic',
                'hebrew',
                'other_afroasiatic',
                'other_western_africa',
                'other_central_eastern_southern_africa',
                'navajo',
                'other_native_north_america',
                'other_unspecified'
            ]
        })
        for groupset, group_list in groupsets.items():
            df[f'total_{groupset}'] = df[
                [f'total_{f}' for f in group_list]
            ].sum(axis=1)
            df[f'{groupset}_and_english_very_well'] = df[
                [f'{f}_and_english_very_well' for f in group_list]
            ].sum(axis=1)
            df[f'{groupset}_and_english_less_than_very_well'] = df[
                [f'{f}_and_english_less_than_very_well' for f in group_list]
            ].sum(axis=1)

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class MedianAgeDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = 'medianage'
    RAW_TABLE_NAME = 'B01002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "median",
        "002E": "male",
        "003E": "female"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
# flake8: NOQA
import logging
from us import states
from .registry import DOWNLOADERS
from .age import AgeDownloader
from .classofworker import ClassOfWorkerDownloader
from .education import EducationDownloader
from .employmentstatus import EmploymentStatusDownloader
from .foreignborn import ForeignBornDownloader
from .householdincome import (
    HouseholdIncomeDownloader,
    HouseholdIncomeLatinoDownloader,
    HouseholdIncomeWhiteDownloader,
    HouseholdIncomeBlackDownloader,
    HouseholdIncomeAsianDownloader
)
from .internet import InternetDownloader
from .language import LanguageDownloader
from .medianage import MedianAgeDownloader
from .mediangrossrent import MedianGrossRentDownloader
from .medianhouseholdincome import (
    MedianHouseholdIncomeDownloader,
    MedianHouseholdIncomeLatinoDownloader,
    MedianHouseholdIncomeWhiteDownloader,
    MedianHouseholdIncomeBlackDownloader,
    MedianHouseholdIncomeAsianDownloader
)
from .medianhousingvalue import MedianHousingValueDownloader
from .medianmonthlyhousingcosts import MedianMonthlyHousingCostsDownloader
from .mobility import (
    MobilityDownloader,
    MobilityBySexDownloader,
    MobilityWhiteDownloader,
    MobilityBlackDownloader,
    MobilityAsianDownloader,
    MobilityLatinoDownloader
)
from .population import PopulationDownloader
from .race import RaceDownloader
from .tenure import (
    TenureDownloader,
    TenureLatinoDownloader,
    TenureWhiteDownloader,
    TenureBlackDownloader,
    TenureAsianDownloader
)
from .yearstructurebuilt import YearStructureBuiltDownloader
logger = logging.getLogger(__name__)


def download_usa(*args, **kwargs):
    """
    Download all the nationwide data.
    """
    logger.debug("Downloading all datasets for all nationwide geographies")
    for klass in DOWNLOADERS:
        obj = klass(*args, **kwargs)
        logger.debug(f"Downloading nationwide {klass.PROCESSED_TABLE_NAME} datasets")
        obj.download_usa()


def download_everything(*args, **kwargs):
    """
    Download all the data.
    """
    logger.debug("Downloading all datasets for all geographies")
    for klass in DOWNLOADERS:
        obj = klass(*args, **kwargs)
        logger.debug(f"Downloading nationwide {klass.PROCESSED_TABLE_NAME} dataset")
        obj.download_usa()
        for state in states.STATES:
            logger.debug(f"Downloading tract-level {klass.PROCESSED_TABLE_NAME} data in {state.name}")
            obj.download_tracts(state.abbr)
            logger.debug(f"Downloading legislative-district-level {klass.PROCESSED_TABLE_NAME} data in {state.name}")
            obj.download_legislative_districts_upper(state.abbr)
            obj.download_legislative_districts_upper(state.abbr)


__all__ = ("DOWNLOADERS", "download_usa", "download_everything")



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class MedianHouseholdIncomeDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincome"
    RAW_TABLE_NAME = 'B19013'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "median"
    })


@register_downloader
class MedianHouseholdIncomeLatinoDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomelatino"
    RAW_TABLE_NAME = 'B19013I'


@register_downloader
class MedianHouseholdIncomeWhiteDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomewhite"
    RAW_TABLE_NAME = 'B19013H'


@register_downloader
class MedianHouseholdIncomeBlackDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomeblack"
    RAW_TABLE_NAME = 'B19013B'


@register_downloader
class MedianHouseholdIncomeAsianDownloader(MedianHouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "medianhouseholdincomeasian"
    RAW_TABLE_NAME = 'B19013D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class InternetDownloader(BaseDownloader):
    YEAR_LIST = (2017,)
    PROCESSED_TABLE_NAME = "internet"
    RAW_TABLE_NAME = 'B28003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': 'total_households',
        '002E': 'total_with_computer',
        '003E': 'dialup_internet',
        '004E': 'broadband_internet',
        '005E': 'computer_no_internet',
        '006E': 'total_no_computer'
    })

    def _process_raw_data(self, *args, **kwargs):
        df = super()._process_raw_data(*args, **kwargs)
        df['total_with_internet'] = df['dialup_internet'] + df['broadband_internet']
        df['total_no_internet'] = df['computer_no_internet'] + df['total_no_computer']
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class ClassOfWorkerDownloader(BaseDownloader):
    """
    Sex by Class of Worker for the Civilian Employed Population 16 Years and Over.
    """
    PROCESSED_TABLE_NAME = "classofworker"
    RAW_TABLE_NAME = 'B24080'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': "total",
        '002E': "male_total",
        '003E': "male_private_for_profit_wage_and_salary",
        '004E': "male_employee_of_private_company",
        '005E': "male_selfemployed_in_own_incorporated_business",
        '006E': "male_private_not_for_profit_wage_and_salary",
        '007E': "male_local_government",
        '008E': "male_state_government",
        '009E': "male_federal_government",
        '010E': "male_selfemployed_in_own_not_incorporated_business",
        '011E': "male_unpaid_family_workers",
        '012E': "female_total",
        '013E': "female_private_for_profit_wage_and_salary",
        '014E': "female_employee_of_private_company",
        '015E': "female_selfemployed_in_own_incorporated_business",
        '016E': "female_private_not_for_profit_wage_and_salary",
        '017E': "female_local_government",
        '018E': "female_state_government",
        '019E': "female_federal_government",
        '020E': "female_selfemployed_in_own_not_incorporated_business",
        '021E': "female_unpaid_family_workers"
    })

    def _process_raw_data(self, *args, **kwargs):
        df = super()._process_raw_data(*args, **kwargs)

        # Calculate totals for both genders together
        groups = [
            "private_for_profit_wage_and_salary",
            "employee_of_private_company",
            "selfemployed_in_own_incorporated_business",
            "private_not_for_profit_wage_and_salary",
            "local_government",
            "state_government",
            "federal_government",
            "selfemployed_in_own_not_incorporated_business",
            "unpaid_family_workers",
        ]
        for g in groups:
            df[f'total_{g}'] = df[f'male_{g}'] + df[f'female_{g}']

        # Calculate custom group sets
        groupsets = collections.OrderedDict({
            "government": [
                'local_government',
                'state_government',
                'federal_government',
            ],
        })
        for groupset, group_list in groupsets.items():
            df[f'total_{groupset}'] = df[[f'total_{f}' for f in group_list]].sum(axis=1)
            df[f'male_{groupset}'] = df[[f'male_{f}' for f in group_list]].sum(axis=1)
            df[f'female_{groupset}'] = df[[f'female_{f}' for f in group_list]].sum(axis=1)

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class YearStructureBuiltDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "yearstructurebuilt"
    RAW_TABLE_NAME = 'B25034'

    @property
    def RAW_FIELD_CROSSWALK(self):
        if self._year > 2014:
            return collections.OrderedDict({
                '001E': 'housing_units',
                '002E': 'built_2014_or_later',
                '003E': 'built_2010_to_2013',
                '004E': 'built_2000_to_2009',
                '005E': 'built_1990_to_1999',
                '006E': 'built_1980_to_1989',
                '007E': 'built_1970_to_1979',
                '008E': 'built_1960_to_1969',
                '009E': 'built_1950_to_1959',
                '010E': 'built_1940_to_1949',
                '011E': 'built_1929_or_earlier'
            })
        elif self._year > 2011:
            return collections.OrderedDict({
                '001E': 'housing_units',
                '002E': 'built_2010_or_later',
                '003E': 'built_2000_to_2009',
                '004E': 'built_1990_to_1999',
                '005E': 'built_1980_to_1989',
                '006E': 'built_1970_to_1979',
                '007E': 'built_1960_to_1969',
                '008E': 'built_1950_to_1959',
                '009E': 'built_1940_to_1949',
                '010E': 'built_1939_or_earlier'
            })
        else:
            return collections.OrderedDict({
                '001E': 'housing_units',
                '002E': 'built_2005_or_later',
                '003E': 'built_2000_to_2004',
                '004E': 'built_1990_to_1999',
                '005E': 'built_1980_to_1989',
                '006E': 'built_1970_to_1979',
                '007E': 'built_1960_to_1969',
                '008E': 'built_1950_to_1959',
                '009E': 'built_1940_to_1949',
                '010E': 'built_1939_or_earlier'
            })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class EducationDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = 'education'
    RAW_TABLE_NAME = 'B15002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': "total",
        '002E': "male_total",
        '003E': "male_no_schooling",
        '004E': "male_nursery_to_4th_grade",
        '005E': "male_5th_and_6th_grade",
        '006E': "male_7th_and_8th_grade",
        '007E': "male_9th_grade",
        '008E': "male_10th_grade",
        '009E': "male_11th_grade",
        '010E': "male_12th_grade_no_diploma",
        '011E': "male_high_school_graduate",
        '012E': "male_less_than_1_year_college",
        '013E': "male_1_or_more_years_college",
        '014E': "male_associates_degree",
        '015E': "male_bachelors_degree",
        '016E': "male_masters_degree",
        '017E': "male_professional_school_degree",
        '018E': "male_doctorate_degree",
        '019E': "female_total",
        '020E': "female_no_schooling",
        '021E': "female_nursery_to_4th_grade",
        '022E': "female_5th_and_6th_grade",
        '023E': "female_7th_and_8th_grade",
        '024E': "female_9th_grade",
        '025E': "female_10th_grade",
        '026E': "female_11th_grade",
        '027E': "female_12th_grade_no_diploma",
        '028E': "female_high_school_graduate",
        '029E': "female_less_than_1_year_college",
        '030E': "female_1_or_more_years_college",
        '031E': "female_associates_degree",
        '032E': "female_bachelors_degree",
        '033E': "female_masters_degree",
        '034E': "female_professional_school_degree",
        '035E': "female_doctorate_degree",
    })

    def _process_raw_data(self, *args, **kwargs):
        """
        Combine raw age columns into our preferred groupings.
        """
        df = super()._process_raw_data(*args, **kwargs)

        # Calculate totals for both genders together
        groups = [
            "no_schooling",
            "nursery_to_4th_grade",
            "5th_and_6th_grade",
            "7th_and_8th_grade",
            "9th_grade",
            "10th_grade",
            "11th_grade",
            "12th_grade_no_diploma",
            "high_school_graduate",
            "less_than_1_year_college",
            "1_or_more_years_college",
            "associates_degree",
            "bachelors_degree",
            "masters_degree",
            "professional_school_degree",
            "doctorate_degree",
        ]
        for g in groups:
            df[f'total_{g}'] = df[f'male_{g}'] + df[f'female_{g}']

        # Calculate our custom groups
        groupsets = collections.OrderedDict({
            "less_than_high_scool": [
                "no_schooling",
                "nursery_to_4th_grade",
                "5th_and_6th_grade",
                "7th_and_8th_grade",
                "9th_grade",
                "10th_grade",
                "11th_grade",
                "12th_grade_no_diploma"
            ],
            "some_college": [
                "less_than_1_year_college",
                "1_or_more_years_college"
            ],
            "masters_or_higher": [
                "masters_degree",
                "professional_school_degree",
                "doctorate_degree"
            ],
            "bachelors_or_higher": [
                "bachelors_degree",
                "masters_degree",
                "professional_school_degree",
                "doctorate_degree"
            ]
        })
        for groupset, group_list in groupsets.items():
            df[f'total_{groupset}'] = df[[f'total_{f}' for f in group_list]].sum(axis=1)
            df[f'male_{groupset}'] = df[[f'male_{f}' for f in group_list]].sum(axis=1)
            df[f'female_{groupset}'] = df[[f'female_{f}' for f in group_list]].sum(axis=1)

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class TenureDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "tenure"
    RAW_TABLE_NAME = 'B25003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001E': 'housing_units',
        '002E': 'owner_occupied',
        '003E': 'renter_occupied'
    })


@register_downloader
class TenureLatinoDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenurelatino"
    RAW_TABLE_NAME = 'B25003I'


@register_downloader
class TenureWhiteDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenurewhite"
    RAW_TABLE_NAME = 'B25003H'


@register_downloader
class TenureBlackDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenureblack"
    RAW_TABLE_NAME = 'B25003B'


@register_downloader
class TenureAsianDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenureasian"
    RAW_TABLE_NAME = 'B25003D'



from setuptools import setup


setup(
    name='census-data-downloader',
    version='0.0.6',
    description="Download Census data and reformat it for humans",
    author='Ben Welsh',
    author_email='ben.welsh@gmail.com',
    url='http://www.github.com/datadesk/census-data-downloader',
    license="MIT",
    packages=("census_data_downloader",),
    install_requires=(
        "pandas",
        "us",
        "census",
        "click"
    ),
    entry_points="""
        [console_scripts]
        censusdatadownloader=census_data_downloader.cli:cmd
    """,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
    ],
)



#! /usr/bin/env python
# -*- coding: utf-8 -*
import os
import time
import logging
import pathlib
import pandas as pd
from us import states
from census import Census
logger = logging.getLogger(__name__)


class BaseDownloader(object):
    """
    Downloads and processes ACS tables from the Census API.
    """
    THIS_DIR = pathlib.Path(__file__).parent
    YEAR_LIST = (
        2017,
        2016,
        2015,
        2014,
        2013,
        2012,
        2011,
        2010,
        2009
    )

    def __init__(
        self,
        api_key=None,
        source="acs5",
        years=None,
        data_dir=None,
        force=False
    ):
        """
        Configuration.
        """
        # Set the inputs
        self.CENSUS_API_KEY = os.getenv("CENSUS_API_KEY", api_key)
        if not self.CENSUS_API_KEY:
            raise NotImplementedError("Census API key required. Pass it as the first argument.")
        self.source = source
        self.force = force

        #
        # Allow custom years for data download, defaulting to most recent year
        #

        # If they want all the years, give it to them.
        if years == "all":
            self.years_to_download = self.YEAR_LIST
        # If the user provides a year give them that.
        elif isinstance(years, int):
            self.years_to_download = [years]
        # Or if they provide years as a list, give those then.
        elif isinstance(years, list):
            self.years_to_download = list(map(int, years))
        # If they provided nothing, default to the latest year of data
        elif years is None:
            self.years_to_download = (max(self.YEAR_LIST),)

        # Validate the years
        for year in self.years_to_download:
            if year not in self.YEAR_LIST:
                error_msg = ("Data only available for the years "
                             f"{self.YEAR_LIST[-1]}-{self.YEAR_LIST[0]}.")
                raise NotImplementedError(error_msg)

        # Set the data directories
        if data_dir:
            self.data_dir = pathlib.Path(str(data_dir))
        else:
            self.data_dir = self.THIS_DIR.joinpath("data")
        self.raw_data_dir = self.data_dir.joinpath("raw")
        self.processed_data_dir = self.data_dir.joinpath("processed")

        # Make sure they exist
        if not self.data_dir.exists():
            self.data_dir.mkdir()
        if not self.raw_data_dir.exists():
            self.raw_data_dir.mkdir()
        if not self.processed_data_dir.exists():
            self.processed_data_dir.mkdir()

    @property
    def censusreporter_url(self):
        """
        Returns the URL of the Census Reporter page explaining the ACS table.
        """
        return f"https://censusreporter.org/tables/{self.RAW_TABLE_NAME}/"

    def download_nationwide(self):
        """
        Download nationwide data.
        """
        csv_suffix = "nationwide"
        api_filter = {'for': 'us:1'}
        geoid_function = lambda row: 1
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_states(self):
        """
        Download data for all states.
        """
        csv_suffix = "states"
        api_filter = {'for': 'state:*'}
        geoid_function = lambda row: row['state']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_congressional_districts(self):
        """
        Download data for all Congressional districts.
        """
        csv_suffix = "congressionaldistricts"
        api_filter = {'for': 'congressional district:*'}
        geoid_function = lambda row: row['state'] + row['congressional district']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_counties(self):
        """
        Download data for all counties.
        """
        csv_suffix = "counties"
        api_filter = {'for': 'county:*'}
        geoid_function = lambda row: row['state'] + row['county']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_places(self):
        """
        Download data for all Census designated places.
        """
        csv_suffix = "places"
        api_filter = {'for': 'place:*'}
        geoid_function = lambda row: row['state'] + row['place']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_tracts(self, state):
        """
        Download data for all Census tracts in the provided state.
        """
        state_obj = getattr(states, state.upper())
        csv_suffix = f"tracts_{state_obj.abbr.lower()}"
        api_filter = {
            'for': 'tract:*',
            'in': f'state: {state_obj.fips}'
        }
        geoid_function = lambda row: row['state'] + row['county'] + row['tract']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_state_legislative_districts_upper(self, state):
        """
        Download data for all Census upper legislative districts in the provided state.
        """
        state_obj = getattr(states, state.upper())
        csv_suffix = f'statelegislativedistrictsupper_{state_obj.abbr.lower()}'
        api_filter = {
            'for': 'state legislative district (upper chamber):*',
            'in': f'state: {state_obj.fips}'
        }
        geoid_function = lambda row: row['state'] + row['state legislative district (upper chamber)']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_state_legislative_districts_lower(self, state):
        """
        Download data for all Census lower legislative districts in the provided state.
        """
        state_obj = getattr(states, state.upper())
        csv_suffix = f'statelegislativedistrictslower_{state_obj.abbr.lower()}'
        api_filter = {
            'for': 'state legislative district (lower chamber):*',
            'in': f'state: {state_obj.fips}'
        }
        geoid_function = lambda row: row['state'] + row['state legislative district (lower chamber)']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_msas(self):
        """
        Download data for Metropolitian Statistical Areas.
        """
        csv_suffix = "msa"
        api_filter = {'for': 'metropolitan statistical area/micropolitan statistical area:*'}
        geoid_function = lambda row: row['metropolitan statistical area/micropolitan statistical area']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_csas(self):
        """
        Download data for Combined Statistical Areas.
        """
        csv_suffix = "csa"
        api_filter = {'for': 'combined statistical area:*'}
        geoid_function = lambda row: row['combined statistical area']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_aiann_homelands(self):
        """
        Download data for American Indian home lands.
        """
        csv_suffix = "aiannhhomeland"
        api_filter = {'for': 'american indian area/alaska native area/hawaiian home land:*'}
        geoid_function = lambda row: row['american indian area/alaska native area/hawaiian home land']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_zctas(self):
        """
        Download data for Zip Code Tabulation Areas
        """
        csv_suffix = "zcta"
        api_filter = {'for': 'zip code tabulation area:*'}
        geoid_function = lambda row: row['zip code tabulation area']
        self._download_tables(api_filter, csv_suffix, geoid_function)

    def download_usa(self):
        """
        Download all datasets that provide full coverage for the entire country.
        """
        self.download_nationwide()
        self.download_states()
        self.download_congressional_districts()
        self.download_counties()
        self.download_places()
        self.download_msas()
        self.download_csas()
        self.download_aiann_homelands()
        self.download_zctas()

    def download_everything(self):
        """
        Download 'em all.
        """
        self.download_usa()
        for state in states.STATES:
            self.download_tracts(state.abbr)
            self.download_state_legislative_districts_upper(state.abbr)
            self.download_state_legislative_districts_lower(state.abbr)

    def _download_tables(self, api_filter, csv_suffix, geoid_function):
        """
        Download and process data.
        """
        for year in self.years_to_download:
            # Set the year as a sneaky private variable we can access elsewhere
            self._year = year

            # Get the raw table
            raw_table = self._get_raw_table(year, api_filter, csv_suffix)

            # Figure out where to write it
            csv_name = f"{self.source}_{year}_{self.PROCESSED_TABLE_NAME}_{csv_suffix}.csv"
            csv_path = self.processed_data_dir.joinpath(csv_name)
            if csv_path.exists() and not self.force:
                logger.debug(f"Processed CSV already exists at {csv_path}")
                continue

            # Process it
            processed_table = self._process_raw_data(raw_table)

            # Add a combined GEOID column with a Census unique identifer
            processed_table['geoid'] = processed_table.apply(geoid_function, axis=1)

            # Write it out
            logger.debug(f"Writing CSV to {csv_path}")
            processed_table.set_index(["geoid", "name"]).to_csv(
                csv_path,
                index=True,
                encoding="utf-8"
            )

    def _get_raw_table(self, year, api_filter, csv_suffix):
        """
        Retrieve the Census API data and return it as an dataframe.
        """
        # Set the CSV name
        csv_name = f"{self.source}_{year}_{self.PROCESSED_TABLE_NAME}_{csv_suffix}.csv"
        csv_path = self.raw_data_dir.joinpath(csv_name)

        # Skip hitting the API if we've already got the file, as long as we're not forcing it.
        if csv_path.exists() and not self.force:
            logger.debug(f"Raw CSV already exists at {csv_path}")
            return csv_path

        # Set the fields we want to pull from API
        field_names = [f"{self.RAW_TABLE_NAME}_{f}" for f in self.RAW_FIELD_CROSSWALK.keys()]
        field_list = ['NAME'] + field_names

        # Hit the API
        logger.debug(f"Downloading raw {self.source} {self.RAW_TABLE_NAME} table for {year} filtered to {api_filter}")
        self.api = getattr(Census(self.CENSUS_API_KEY, year=year), self.source)
        data = self.api.get(field_list, api_filter)

        # Convert it to a DataFrame
        df = pd.DataFrame.from_records(data)

        # Rename the ugly NAME field
        df.rename(columns={"NAME": "name"}, inplace=True)

        # Write it to disk
        logger.debug(f"Writing raw ACS table to {csv_path}")
        df.to_csv(csv_path, index=False, encoding="utf-8")

        # Pause to be polite to the API.
        time.sleep(1)

        # Return the path
        return csv_path

    def _process_raw_data(self, path):
        """
        Accepts a path to a CSV with raw ACS data from the Census API.

        Returns it polished for human analysis.
        """
        # Read in the raw CSV of data from the ACS
        df = pd.read_csv(
            path,
            dtype={
                "state": str,
                "county": str,
                "place": str,
                "tract": str,
                "congressional district": str,
                "state legislative district (upper chamber)": str,
                "state legislative district (lower chamber)": str
            }
        )

        # Rename fields with humanized names
        field_name_mapper = dict(
            (f"{self.RAW_TABLE_NAME}_{raw}", processed)
            for raw, processed in self.RAW_FIELD_CROSSWALK.items()
        )
        df.rename(columns=field_name_mapper, inplace=True)

        # Cast numbers to integers
        for field in field_name_mapper.values():
            df[field].astype(pd.np.float64)

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from .base import BaseDownloader
from .registry import register_downloader


@register_downloader
class MedianHousingValueDownloader(BaseDownloader):
    PROCESSED_TABLE_NAME = "medianhousingvalue"
    RAW_TABLE_NAME = 'B25077'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001E": "median"
    })
